@@ -14,7 +14,7 @@ class ExposureSpecification:
     """
 
     def __init__(self, pattern, ngroup, nint, nexp, tframe, nframe=1, nsample=1, nsample_skip=0, tfffr=0,
-                 subarray='Full', nskip=0, nprerej=0, npostrej=0, nreset1=1, ndrop1=0, ndrop3=0, nreset2=1,
+                 subarray='Full', ndrop2=0, nprerej=0, npostrej=0, nreset1=1, ndrop1=0, ndrop3=0, nreset2=1,
                  frame0=False, det_type='h2rg'):
         """
         Create a generic Exposure Specification.
@@ -54,10 +54,6 @@ class ExposureSpecification:
             name of subarray to be read.
             If None, readout is full frame
 
-        nskip: int (optional, default 0)
-            number of skipped frames per group. Only
-            supported by some readout patterns.
-
         nprerej: int (optional, default 0)
             The MIR detectors (i.e. MIRI) may reject the first and/or last groups of a ramp.
 
@@ -70,6 +66,10 @@ class ExposureSpecification:
         ndrop1: int (optional, default 0)
             Number of frames dropped at the beginning of an integration
 
+        ndrop2: int (optional, default 0)
+            number of skipped frames per group. Only
+            supported by some readout patterns.
+
         ndrop3: int (optional, default 0)
             Number of frames dropped at the end of an integration
 
@@ -100,12 +100,12 @@ class ExposureSpecification:
         # Optional parameters
         self.nframe = nframe
         self.subarray = subarray
-        self.nskip = nskip
         self.nprerej = nprerej
         self.npostrej = npostrej
         self.nreset1 = nreset1
         self.nreset2 = nreset2
         self.ndrop1 = ndrop1
+        self.ndrop2 = ndrop2
         self.ndrop3 = ndrop3
         self.nsample = nsample
         self.nsample_skip = nsample_skip
@@ -135,25 +135,28 @@ class ExposureSpecification:
         """
         The time formulae are defined in Valenti et al. 2017, JWST-STScI-006013
 
+        A small alteration has been made: the nskip parameter is now ndrop2, to follow 
+        the pending format from Holler et al. 2021, JWST-STScI-006013-A
+
         The equations in this method are duplicated in the front-end
         (workbook.js, update_detector_time_labels function). Please ensure
         that changes to these equations are reflected there.
         """
-        self.tgroup = self.tframe * (self.nframe + self.nskip)
+        self.tgroup = self.tframe * (self.nframe + self.ndrop2)
 
         # Equation 4
-        self.measurement_time = self.tframe * self.nint * ((self.ngroup - 1) * (self.nframe + self.nskip))
+        self.measurement_time = self.tframe * self.nint * ((self.ngroup - 1) * (self.nframe + self.ndrop2))
         # Equation 5
         if self.frame0:
             self.measurement_time += 0.5 * self.nint * self.tframe * self.nframe
 
         # Equation 1
         self.exposure_time = (self.tfffr * self.nint) + self.tframe * (self.nreset1 + (self.nint - 1) * self.nreset2 +
-                                self.nint * (self.ndrop1 + (self.ngroup - 1) * (self.nframe + self.nskip) +
+                                self.nint * (self.ndrop1 + (self.ngroup - 1) * (self.nframe + self.ndrop2) +
                                 self.nframe + self.ndrop3))
         # Equation 7
         self.saturation_time = self.tfffr + self.tframe * (self.ndrop1 +
-                               (self.ngroup - 1) * (self.nframe + self.nskip) + self.nframe)
+                               (self.ngroup - 1) * (self.nframe + self.ndrop2) + self.nframe)
 
         self.duty_cycle = self.measurement_time / self.exposure_time
         self.total_exposure_time = self.nexp * self.exposure_time
@@ -161,32 +164,31 @@ class ExposureSpecification:
 
     def get_times_mir(self):
         """
-        The time formulae are defined in Valenti et al. 2017, JWST-STScI-006013. Note that we have to subtract the
+        The time formulae are defined in Holler et al. 2021, JWST-STScI-006013-A. Note that we have to subtract the
         groups that are rejected (by the pipeline) from the measurement time (nprerej+npostrej). The saturation time
         conservatively considers the ramp saturated even if saturation occurs in a rejected frame.
 
+        Note that these equations are generic, suitable for both H2RG and SiAs detectors.
+
         The equations in this method are duplicated in the front-end
         (workbook.js, update_detector_time_labels function). Please ensure
         that changes to these equations are reflected there.
         """
+        self.tgroup = self.tframe * (self.nframe + self.ndrop2)
 
-        self.tgroup = self.tframe * (self.nframe + self.nskip)
-
-        # Valenti et al. 2017 defines the very bright regime for MIRI as ngroup>=5.
-        # This could eventually be pulled out to data, rather than hardcoded.
-        # Equation 6
-        if self.ngroup >= 5:
-            self.measurement_time = self.nint * (self.ngroup - 1 - (self.nprerej + self.npostrej)) * self.tgroup
-        else:
-            # very bright regime
-            # Equation 10
-            self.measurement_time = self.nint * (self.tfffr + self.ngroup * self.tgroup) \
-                - self.nint * self.tframe * 0.5 * (self.nframe - 1)
-
+        # MIRI measurement time for ngroups < 5 is now handled with dropframes
         # Equation 3
-        self.exposure_time = self.nint * (self.tfffr + self.ngroup * self.nframe * self.tframe)
-        # Equation 8
-        self.saturation_time = self.tfffr + self.ngroup * self.nframe * self.tframe
+        self.measurement_time = self.nint * self.tframe * (self.ngroup - 1 - self.nprerej - self.npostrej) * (self.nframe + self.ndrop2)
+         # Equation 4
+        if self.frame0:
+            self.measurement_time += 0.5 * self.nint * self.tframe * (self.nframe - 1)
+
+        # Equation 1
+        self.exposure_time = (self.tfffr * self.nint) + self.tframe * (self.nreset1 + (self.nint - 1) * self.nreset2 +
+                                self.nint * (self.ndrop1 + (self.ngroup - 1) * (self.nframe + self.ndrop2) +
+                                self.nframe + self.ndrop3))
+        # Equation 6
+        self.saturation_time = self.tframe * (self.ndrop1 + (self.ngroup - 1) * (self.nframe + self.ndrop2) + self.nframe)
 
         self.duty_cycle = self.measurement_time / self.exposure_time
         self.total_exposure_time = self.nexp * self.exposure_time
@@ -239,8 +241,8 @@ class ExposureSpecification:
         ngroup. The resulting fractional value is then rounded down to an integer.
 
         These equations are derived by setting saturation_time = time_to_saturation in equation 7 (for NIR H2RG
-        detectors) or equation 8 (for MIR SiAs detectors) of the timing document, Valenti et al. 2017,
-        JWST-STScI-006013, and solving for n_groups.
+        detectors) or equation 6 (for MIR SiAs detectors, now out of the timing document revision, JWST-STScI-006013-A)
+        of the timing document, Valenti et al. 2017, JWST-STScI-006013, and solving for n_groups.
 
         Technically speaking, we are getting the groups BEFORE saturation by rounding the number of groups down to the
         nearest integer. If groups_before_sat is EXACTLY an integer (which is unlikely), we will get one partially
@@ -258,10 +260,13 @@ class ExposureSpecification:
         # we clip to 1e-10 to avoid division by zero errors even if the slope is tiny.
         time_to_saturation = fullwell / slope.clip(MIN_CLIP, np.max([MIN_CLIP, np.max(slope)]))
         if self.det_type == 'sias':
-            groups_before_sat = (time_to_saturation - self.tfffr) / (self.nframe * self.tframe)
+            groups_before_sat = ((time_to_saturation / self.tframe) - self.nframe - self.ndrop1) / \
+                                (self.nframe + self.ndrop2) + 1 
         elif self.det_type == 'h2rg':
             groups_before_sat = (((time_to_saturation - self.tfffr) / self.tframe) - self.nframe) / (self.nframe +
-                                                                                                     self.nskip) + 1.
+                                                                                                     self.ndrop2) + 1.
+        
+
         else:
             raise ValueError('Unknown detector type %s' % (self.det_type))
         # The groups_before_sat value should never be less than 0 - that would imply saturation before we started
@@ -338,9 +343,8 @@ class ExposureSpecification:
         # removing with any pre- and post-rejected groups here. This is not stictly directly related to saturation, bu
         # behaves in a similar way (if there is <2 groups available, we cannot define a slope). The very bright regime
         # in which groups are not rejected can be implemented (in parts) by setting nprerej=npostrej=0.
-        # Important: frames are not rejected in the Very Bright Regime (self.ngroup<5).
         # Therefore we disable it here in that regime.
-        if ((self.nprerej != 0) or (self.npostrej != 0)) and (self.ngroup>=5):
+        if ((self.nprerej != 0) or (self.npostrej != 0)):
             # If a group is already rejected because of saturation, there is no need to reject another at the end of
             # the ramp.
             unsat_pixels = (self.ngroup-np.ceil(n))<self.npostrej
@@ -464,7 +468,7 @@ class ExposureSpecificationTA(ExposureSpecification):
     """
 
     def __init__(self, pattern, ngroup, ngroup_extract, nint, nexp, tframe, nframe=1, nsample=1, tfffr=0,
-                 nsample_skip=1, subarray='Full', nskip=0, nprerej=0, npostrej=0, nreset1=1, ndrop1=0, ndrop3=0,
+                 nsample_skip=1, subarray='Full', ndrop2=0, nprerej=0, npostrej=0, nreset1=1, ndrop1=0, ndrop3=0,
                  nreset2=1, det_type='h2rg'):
         """
         Create a Target Acquisition Exposure Specification.
@@ -505,8 +509,8 @@ class ExposureSpecificationTA(ExposureSpecification):
             name of subarray to be read.
             If None, readout is full frame
 
-        nskip: int (optional, default 0)
-            number of skipped frames per group. Only
+        ndrop2: int (optional, default 0)
+            number of dropped frames per group. Only
             supported by some readout patterns.
 
         nprerej: int (optional, default 0)
@@ -526,7 +530,7 @@ class ExposureSpecificationTA(ExposureSpecification):
         """
 
         ExposureSpecification.__init__(self, pattern, ngroup, nint, nexp, tframe, nframe=nframe,
-                 subarray=subarray, nskip=nskip, nsample_skip=nsample_skip, tfffr=tfffr,
+                 subarray=subarray, ndrop2=ndrop2, nsample_skip=nsample_skip, tfffr=tfffr,
                  nprerej=nprerej, npostrej=npostrej, nreset1=nreset1, ndrop1=ndrop1, ndrop3=ndrop3,
                  nreset2=nreset2, det_type=det_type)
 
