@@ -789,7 +789,7 @@ class Instrument(InstrumentConfig):
             tframe = self.subarray_config[readout_pattern][subarray]["tframe"]
             tfffr = self.subarray_config[readout_pattern][subarray]["tfffr"]
         nframe = self.readout_pattern_config[readout_pattern]["nframe"]
-        nskip = self.readout_pattern_config[readout_pattern]["nskip"]
+        ndrop2 = self.readout_pattern_config[readout_pattern]["ndrop2"]
 
         # If these are trivial, we don't have to define them.
         if "nsample" in self.readout_pattern_config[readout_pattern]:
@@ -811,6 +811,9 @@ class Instrument(InstrumentConfig):
             if "nreset1" in self.subarray_config[readout_pattern][subarray]:
                 nreset1 = self.subarray_config[readout_pattern][subarray]["nreset1"]
                 nreset2 = self.subarray_config[readout_pattern][subarray]["nreset2"]
+        if "nreset1" in self.readout_pattern_config[readout_pattern]:
+            nreset1 = self.readout_pattern_config[readout_pattern]["nreset1"]
+            nreset2 = self.readout_pattern_config[readout_pattern]["nreset2"]
 
 
         # These are defined in the reference data as "ramp_config" and apply to a whole instrument
@@ -832,7 +835,7 @@ class Instrument(InstrumentConfig):
                 tframe,
                 nframe=nframe,
                 subarray=subarray,
-                nskip=nskip,
+                ndrop2=ndrop2,
                 nsample=nsample,
                 nsample_skip=nsample_skip,
                 nprerej=nprerej,
@@ -853,7 +856,7 @@ class Instrument(InstrumentConfig):
                 tframe,
                 nframe=nframe,
                 subarray=subarray,
-                nskip=nskip,
+                ndrop2=ndrop2,
                 nsample=nsample,
                 nsample_skip=nsample_skip,
                 nprerej=nprerej,
@@ -919,7 +922,8 @@ class Instrument(InstrumentConfig):
             'aperture': self.instrument['aperture'],
             'subarray': self.detector['subarray'],
             'readout_pattern': self.detector['readout_pattern'],
-            'instmode': self.instrument['mode']
+            'instmode': self.instrument['mode'],
+            'ngroup': self.detector['ngroup']
         }
 
         if parameter_key in item:
@@ -936,13 +940,22 @@ class Instrument(InstrumentConfig):
                 #     "sw": 17
                 # },
 
+                # start with the default; if nothing else applies we'll return that value
+                if "default" in item[parameter_key]:
+                    parameter_value = item[parameter_key]['default']
+                else:
+                    raise DataError("There is no default value in {}".format(item[parameter_key]))
                 # Create the key lookup
                 key_lookup = []
                 for kf in item[parameter_key]['key_format']:
                     kfelement = kf.split('__')
                     for kfe in kfelement:
                         try:
-                            key_lookup.append(value_lookup[kfe])
+                            key_lookup.append(str(value_lookup[kfe]))
+                            # if this value alone is in the item listing, grab it
+                            # (if more than one shows up, there has to be a listing with both)
+                            if str(value_lookup[kfe]) in item[parameter_key]:
+                                parameter_value = item[parameter_key][str(value_lookup[kfe])]
                         except KeyError as e:
                             raise DataError(
                                 "Unknown key format {} value {} listed in {}".format(kfe, value_lookup[kfe],
@@ -957,11 +970,6 @@ class Instrument(InstrumentConfig):
                     if key_lookup in item[parameter_key]:
                         parameter_value = item[parameter_key][key_lookup]
 
-                    # If the key_lookup was not present then just return the default.
-                    elif "default" in item[parameter_key]:
-                        parameter_value = item[parameter_key]['default']
-                    else:
-                        raise DataError("There is no default value in {}".format(item[parameter_key]))
             else:
                 raise DataError("Improperly formatted parameter {}".format(item))
         else:
